unit DBTableGridFrame;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, ComCtrls, ExtCtrls, StdCtrls, Grids, Dialogs,
  fpSpreadsheet, fpsTypes, xlsxOOXML, xlsBiff8, fpsOpenDocument, ZStream;

type

  { TFrame1 }

  TFrame1 = class(TFrame)
    ButtonExport: TButton;
    ButtonToggleQuery: TButton;
    ComboBoxSeparator: TComboBox;
    ComboBoxExportFormat: TComboBox;
    Edit1: TEdit;
    Label1: TLabel;
    Panel: TPanel;
    StatusBar: TStatusBar;
    StringGrid: TStringGrid;
    procedure ButtonExportClick(Sender: TObject);
    procedure ButtonToggleQueryClick(Sender: TObject);
    procedure ComboBoxExportFormatChange(Sender: TObject);
  private

  public
    procedure   WriteCellDataHandler(Sender: TsWorksheet; ARow, ACol: Cardinal; var AData: variant; var AStyleCell: PCell);
    constructor Create(TheOwner: TComponent); override;
  end;

implementation

{$R *.lfm}

{ Helper functions/procedures }
function RandomString(ALength: Integer): String;
var
  i: Integer;
  n: Integer;
begin
  Result := '';
  for i:=1 to ALength do begin
    n := Random(29);
    case n of
      26 : Result := Result + 'ä';
      27 : Result := Result + 'ö';
      28 : Result := Result + 'ü';
      29 : Result := Result + 'ß';
      else Result := Result + char(n + ord('a'));
    end;
  end;
end;

{ TFrame1 }
procedure TFrame1.ComboBoxExportFormatChange(Sender: TObject);
begin
  case ComboBoxExportFormat.ItemIndex of
   // Microsoft Excel (xlsx)
  0: begin
        ComboBoxExportFormat.Hint := 'Microsoft Excel (.xlsx)';
        ComboBoxSeparator.Enabled := false;
        ComboBoxSeparator.Visible := false;
     end;
  // Libre/Open Office Calc (ods)
  1: begin
        ComboBoxExportFormat.Hint := 'Libre/Open Office Calc (.ods)';
        ComboBoxSeparator.Enabled := false;
        ComboBoxSeparator.Visible := false;
     end;
  // CSV
  2: begin
       ComboBoxExportFormat.Hint := 'Comma Separaed Values (.csv)';
       ComboBoxSeparator.Enabled := true;
       ComboBoxSeparator.Visible := true;
     end;
  // ZIP
  3: begin
       ComboBoxExportFormat.Hint := 'Compressed Comma Separaed Values (.zip)';
       ComboBoxSeparator.Enabled := true;
       ComboBoxSeparator.Visible := true;
     end;
  // FPZ
  4: begin
       ComboBoxExportFormat.Hint := 'Compressed Comma Separaed Values (.fpz)';
       ComboBoxSeparator.Enabled := true;
       ComboBoxSeparator.Visible := true;
     end;
  end;
end;

procedure TFrame1.ButtonToggleQueryClick(Sender: TObject);
var
  x, y:      integer;
  ClickedOK: boolean;
  Zeilen:    string;
  t: TDateTime;

begin
  Screen.Cursor := crHourglass;
  StringGrid.Clear;
  Application.ProcessMessages;
  try
    Randomize;
    Zeilen := Edit1.Text;
    t := Now;
    ClickedOK := true;

    if ClickedOK then
    begin
      StringGrid.Columns.Clear;
      for x := StringGrid.FixedCols to (StringGrid.FixedCols + 7) - 1 do
      begin
        StringGrid.Columns.Add;
      end;

      StringGrid.RowCount := StringGrid.FixedRows + StrToIntDef(Zeilen, 1);
      StringGrid.Columns[0].Title.Caption:='No.';
      StringGrid.Columns[1].Title.Caption:='Integer';
      StringGrid.Columns[2].Title.Caption:='Float';
      StringGrid.Columns[3].Title.Caption:='Date';
      StringGrid.Columns[4].Title.Caption:='DateTime';
      StringGrid.Columns[5].Title.Caption:='Time';
      StringGrid.Columns[6].Title.Caption:='String';
      //StringGrid.RowCount := StringGrid.RowCount + StringGrid.FixedRows;
    end;

    (* Testdaten eintragen *)
    (* Zeile *)
    for x := StringGrid.FixedRows to StringGrid.RowCount - 1 do
    begin
      (* Spalte *)
      for y := 0 to (StringGrid.ColCount - 1) do
      begin
        case y of
          0: (* integer *)
          begin
            StringGrid.Cells[y + StringGrid.FixedCols, x] := IntToStr(x);
          end;
          1: (* integer *)
          begin
            StringGrid.Cells[y + StringGrid.FixedCols, x] := IntToStr(Random(10000));
          end;
          2: (* Float *)
          begin
            StringGrid.Cells[y + StringGrid.FixedCols, x] := FormatFloat('0.00', Random(10000) * Random());
          end;
          3: (* Date *)
          begin
            StringGrid.Cells[y + StringGrid.FixedCols, x] := FormatDateTime('dd.mm.yyyy', Date + x);
          end;
          4: (* DateTime *)
          begin
            StringGrid.Cells[y + StringGrid.FixedCols, x] := FormatDateTime('dd.mm.yyyy hh:nn:ss', Date +  Random());
          end;
          5: (* Time *)
          begin
            StringGrid.Cells[y + StringGrid.FixedCols, x] := FormatDateTime('hh:nn:ss', time +  Random());
          end;
          6: (* String *)
          begin
            StringGrid.Cells[y + StringGrid.FixedCols, x] := RandomString(Random(64)+1);
          end;
        end;
      end;
    end;

    StatusBar.SimpleText := 'Füllen dauerte ' + FormatDateTime('hh:nn:ss.zzz', Now-t);
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFrame1.WriteCellDataHandler(Sender: TsWorksheet; ARow, ACol: Cardinal; var AData: variant; var AStyleCell: PCell);
begin
  if (ACol = 0) and (ARow = 0) then
    AData := ''
  else
  if ARow = 0 then
    AData := StringGrid.Columns[ACol-StringGrid.FixedCols].Title.Caption   // Achtung: das wird -1 wenn ACol=0 and StringGrid.FixedCols>0 ist!!!
  else
    AData := StringGrid.Cells[ACol, ARow];
end;

procedure TFrame1.ButtonExportClick(Sender: TObject);
var
  t:                 TDateTime;
  workbook:          TsWorkbook;
  worksheet:         TsWorksheet;
  MemoryStream:      TMemoryStream;
  GZFileStream:      TGZFileStream;
  FilestreamOut:     TFilestream;
  CompressionStream: TCompressionStream;

begin
  Screen.Cursor := crHourglass;
  Application.ProcessMessages;
  t := Now;

  if (StringGrid.RowCount > StringGrid.FixedRows) and (StringGrid.ColCount > StringGrid.FixedCols) then
  begin
    try
      case ComboBoxExportFormat.ItemIndex of
        0: begin
          try
            workbook := TsWorkbook.Create;
            workbook.Options := [boVirtualMode, boBufStream];
            worksheet := workbook.AddWorksheet('Data');
            worksheet.VirtualRowCount := StringGrid.RowCount;
            worksheet.VirtualColCount := StringGrid.ColCount;
            worksheet.OnWriteCellData := @WriteCellDataHandler;
            workbook.WriteToFile('test.xlsx', sfOOXML, true);
            StatusBar.SimpleText := 'XLSX-Export dauerte ' + FormatDateTime('hh:nn:ss.zzz', Now-t) + ' sec.';
          finally
            workbook.Free;
          end;
           end;
        1: begin
             try
               workbook := TsWorkbook.Create;
               workbook.Options := [boVirtualMode, boBufStream];
               worksheet := workbook.AddWorksheet('Data');
               worksheet.VirtualRowCount := StringGrid.RowCount;
               worksheet.VirtualColCount := StringGrid.ColCount;
               worksheet.OnWriteCellData := @WriteCellDataHandler;
               workbook.WriteToFile('test.ods', sfOpenDocument, true);
               StatusBar.SimpleText := 'ODS-Export dauerte ' + FormatDateTime('hh:nn:ss.zzz', Now-t) + ' sec.';
             finally
               workbook.Free;
             end;
           end;
        2: begin
             StringGrid.SaveToCSVFile('test.csv', ComboBoxSeparator.Text[1], true, false);
             StatusBar.SimpleText := 'CSV-Export dauerte ' + FormatDateTime('hh:nn:ss.zzz', Now-t) + ' sec.';
           end;
        3: begin
             MemoryStream := TMemoryStream.Create();
             StringGrid.SaveToCSVStream(MemoryStream, ComboBoxSeparator.Text[1], true, true);
             MemoryStream.Position := 0;
             GZFileStream := TGZFileStream.create('test.csv.gzip', gzopenwrite);
             GZFileStream.CopyFrom(MemoryStream, MemoryStream.Size);
             GZFileStream.Free;
             MemoryStream.Free;

             StatusBar.SimpleText := 'ZIP-Compressed-CSV-Export dauerte ' + FormatDateTime('hh:nn:ss.zzz', Now-t) + ' sec.';
           end;
        4: begin
             MemoryStream  := TMemoryStream.Create();
             FilestreamOut := TFileStream.Create('test.csv.fpz', fmCreate);

             StringGrid.SaveToCSVStream(MemoryStream, ComboBoxSeparator.Text[1], true, true);
             MemoryStream.Position := 0;
             CompressionStream := TCompressionStream.Create(clMax, FilestreamOut);
             CompressionStream.CopyFrom(MemoryStream, MemoryStream.Size);
             CompressionStream.Free;
             FilestreamOut.Free;
             MemoryStream.Free;

             StatusBar.SimpleText := 'FPZ-Compressed-CSV-Export dauerte ' + FormatDateTime('hh:nn:ss.zzz', Now-t) + ' sec.';
        end;
      end;
    finally
      Screen.Cursor := crDefault;
    end;

  end;
  Screen.Cursor := crDefault;
end;

constructor TFrame1.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  ComboBoxExportFormat.ItemIndex := 0;
  ComboBoxExportFormat.Hint := 'Microsoft Excel (.xlsx)';
  ComboBoxSeparator.Hint    := 'Separator for .csv an .txt files';
  ComboBoxSeparator.Enabled := false;
  ComboBoxSeparator.Visible := false;
end;

end.

